# Version 2.00, last modified on January 16, 2013 by Ryan Ly

# constants
w.min <- 0
w.max <- 1e7

# The weber model eqn: given a Weber fraction and a list of ratios for N trials,
# compute the probabilities of getting the N trials correct.
weber.model <- function(w, r) {
	return(pnorm((r-1)/(w*sqrt(1+r^2))))
}

# Find w by minimizing least-squares error, using nls algorithm (Gauss-Newton)
# default.
fit.w.ls <- function(r, y) {
  w.nls.start <- 1.0  # nls' starting w value
  w.nls.decrement <- 0.3  # for backup w values
  w.nls <- w.nls.start
  d.temp <- data.frame(r=r, y=y)
  H<-NA  # the model information
  while (is.na(H)[1] && w.nls > w.min) {
      	xx <- try(H <- nls(y ~ weber.model(w, r), 
                       data=d.temp, 
                       start=list(w=w.nls)))
    if (inherits(xx, "try-error")) {
      H <- NA
      w.nls <- w.nls - w.nls.decrement  # decrement starting w if failed
    }
  }
  if (is.na(H)[1]) {
    cat("-- could not solve the nonlinear model (w=NA)\n")
  }
  return(H)	 # return the NLS object. call coef(ret)[[1]] to get value
}

# Generate matrix holding average %corr and stdev for each ratio
# Uses unique ratios of the input
make.bin.ratio.matrix <- function(r, y, unique.ratios) {
  dt <- data.frame(r=r, y=y)
  U <- unique(dt$r)
  M <- matrix(0, nrow=length(unique.ratios), ncol=5)
  colnames(M) <- c("ratio", "ntrials", "ncorrect", "pest", "sd")
  row <- 0
  for (u in unique.ratios) {			# for each bin
    row <- row + 1
    if (u %in% U) {
      J <- dt$r == u  # trial indices for this bin
      M[row,1] <- u  # center of ratio bin
      n <- dim(dt[J,])[1]
      M[row,2] <- n  # num trials
      M[row,3] <- sum(dt$y[J])  # num correct
      p <- M[row,3]/n
      M[row,4] <- p  # % correct (p est)
      M[row,5] <- sqrt(p*(1-p)/n)  # stdev
    }
    else
      M[row,] <- rep(NA,5)
  }
  return(M)
}

# Cutoff values for the non-recursive Jolicouer outlier method
# See: Van Selts, M. & Jolicoeur, P. (1994). A solution to the effect 
# of sample size on outlier elimination. Quarterly Journal of 
# Experimental Psychology, 47A, 631-650.
#
# If a data point is jol.cutoff[n] standard deviations from the mean,
# it is considered an outlier, where n is the number of data points.
jol.cutoff <- rep(NA, 100)
jol.cutoff[4:15] <- c(1.458, 1.68, 1.841, 1.961, 2.05, 2.12, 2.173, 2.22,
                      2.246, 2.274, 2.31, 2.326)
i<-16:20; jol.cutoff[i] <- ((2.391-2.326) * ((i-15)/5)) + 2.326
i<-21:25; jol.cutoff[i] <- ((2.410-2.391) * ((i-20)/5)) + 2.391
i<-26:30; jol.cutoff[i] <- ((2.4305-2.410) * ((i-25)/5)) + 2.410
i<-31:35; jol.cutoff[i] <- ((2.45-2.4305)  * ((i-30)/5)) + 2.4305
i<-36:50; jol.cutoff[i] <- ((2.48-2.45) * ((i-35)/15)) + 2.45
i<-51:100; jol.cutoff[i] <- ((2.50-2.48) * ((i-50)/50)) + 2.48

# Returns an array of T/F indicating which elements of the passed
# array to REMOVE based on the non-recursive jolicouer outlier method. 
# F values are the elements to keep.
# If there are fewer than 4 trials, no trials are deemed outliers.
find.outliers.jolicoeur <- function(arr) {
  arr.n <- length(arr)
  arr.mean <- mean(arr)
  arr.sd <- sd(arr)
  arr.devs <- abs(arr-arr.mean)/arr.sd  # num sd's from the mean 
  if (arr.n > 100)
    cutoff <- 2.5
  else if (arr.n < 4)
    cutoff <- Inf  # n too small, don't cut out any values
  else
    cutoff <- jol.cutoff[arr.n]
  return(arr.devs >= cutoff)
}
